CREATE OR REPLACE PACKAGE intranet AS
	Type NbrType IS RECORD (
		NUMBER_DATA	NUMBER);
	Type ProcType IS RECORD (
		PROCEDURE_NAME	VARCHAR2(30));
	Type UserRecType IS RECORD (
		FNAME	VARCHAR2(20),
		SNAME	VARCHAR2(15),
		SITE	NUMBER(4));
	Type CallinfoRecType IS RECORD (
		CALL_NO		NUMBER,
		TAKEN		VARCHAR2(10),
		PRIORITY	VARCHAR2(10),
		NAME		VARCHAR2(35),
		PROBLEM		VARCHAR2(240));
	Type CalllogRecType IS RECORD (
		NBR		NUMBER,
		PHONE		VARCHAR2(14),
		FNAME		VARCHAR2(35),
		SNAME		VARCHAR2(35),
		SITE		VARCHAR2(10),
		PRIORITY	VARCHAR2(10),
		CATEGORY	VARCHAR2(10),
		STATUS		VARCHAR2(10),
		PROBLEM		VARCHAR2(240),
		SOLUTION	VARCHAR2(240),
		TAKEN_DATE	VARCHAR2(10),
		TAKEN_TIME	VARCHAR2(8),
		TAKEN_BY	VARCHAR2(15),
		ASSIGNED_DATE	VARCHAR2(10),
		ASSIGNED_TIME	VARCHAR2(8),
		ASSIGNED_BY	VARCHAR2(15),
		ASSIGNED_TO	VARCHAR2(10),
		COMPLETED_DATE	VARCHAR2(10),
		COMPLETED_TIME	VARCHAR2(8),
		COMP_BY		VARCHAR2(15));

--	**********************************************************************
	Type RoomRecType IS RECORD (
		TIME	VARCHAR2(8),
		MESG	VARCHAR2(60));

	Type RoomRecCurType IS REF CURSOR RETURN RoomRecType;


	Type CallinfoRecCurType IS REF CURSOR RETURN CallinfoRecType;
	Type CalllogRecCurType IS REF CURSOR RETURN CalllogRecType;
	Type ProcCurType IS REF CURSOR RETURN ProcType;
	Type ProcArgCurType IS REF CURSOR RETURN all_arguments%ROWTYPE;
	Type UserRecCurType IS REF CURSOR RETURN UserRecType;
	Type PhoneRecCurType IS REF CURSOR RETURN whitepages%ROWTYPE;
	Type NbrCurType IS REF CURSOR RETURN NbrType;

	FUNCTION get_procs (application VARCHAR2) RETURN ProcCurType;
	FUNCTION get_proc_args (application VARCHAR2) RETURN ProcArgCurType;
	FUNCTION helpdesk_callno_query RETURN NbrCurType;
	PROCEDURE get_time (time_string OUT VARCHAR2);
	PROCEDURE get_date (date_string OUT VARCHAR2);
	FUNCTION helpdesk_time_query (dummy IN VARCHAR2) RETURN ProcCurType;
	FUNCTION helpdesk_date_query (dummy IN VARCHAR2) RETURN ProcCurType;
	FUNCTION helpdesk_phone_query (phone_no IN VARCHAR2) RETURN UserRecCurType;
	FUNCTION helpdesk_callinfo_query (choice IN NUMBER, helpdesk_dude IN VARCHAR2) RETURN CallinfoRecCurType;
	FUNCTION helpdesk_calllog_query (log_no IN NUMBER) RETURN CalllogRecCurType;
	PROCEDURE helpdesk_calllog_add (log_no IN NUMBER, phone IN VARCHAR2, priority IN VARCHAR2, category IN VARCHAR2, 
		status IN VARCHAR2, assigned IN VARCHAR2, problem IN VARCHAR2, taken_by IN VARCHAR2);
	PROCEDURE helpdesk_phone_add (phone_no IN VARCHAR2, fname IN VARCHAR2, sname IN VARCHAR2, site_in IN NUMBER);
	PROCEDURE helpdesk_calllog_update (nbr IN NUMBER, priority IN VARCHAR2, category IN VARCHAR2, status IN VARCHAR2, 
		problem IN VARCHAR2, solution IN VARCHAR2, assigned_date IN VARCHAR2, assigned_time IN VARCHAR2, 
		assigned_by IN VARCHAR2, assigned_to IN VARCHAR2, completed_date IN VARCHAR2, completed_time IN VARCHAR2,
		comp_by IN VARCHAR2);
	PROCEDURE helpdesk_calllog_delete (nbr IN NUMBER);
	FUNCTION phone_query (choice IN NUMBER, search_string IN VARCHAR2) RETURN PhoneRecCurType;
	PROCEDURE phone_add (sname IN VARCHAR2, fname IN VARCHAR2, job_title IN VARCHAR2, phone_no IN VARCHAR2, 
		mobile_no in VARCHAR2, site_in IN NUMBER);
	PROCEDURE phone_update (nbr IN NUMBER, sname IN VARCHAR2, fname IN VARCHAR2, job_title IN VARCHAR2, phone_no IN VARCHAR2, 
		mobile_no in VARCHAR2, site_in IN VARCHAR2, offic IN VARCHAR2);
	PROCEDURE phone_delete (nbr IN NUMBER);

--	**********************************************************************
	FUNCTION roombook_query (rbi_room IN VARCHAR2, rbi_date in VARCHAR2) RETURN RoomRecCurType;
	PROCEDURE roombook_add (rbi_room IN VARCHAR2, rbi_date IN VARCHAR2, rbi_time IN VARCHAR2, rbi_mesg IN VARCHAR2);
	PROCEDURE roombook_update (rbi_room IN VARCHAR2, rbi_date IN VARCHAR2, rbi_time IN VARCHAR2, rbi_mesg IN VARCHAR2);
	PROCEDURE roombook_delete (rbi_room IN VARCHAR2, rbi_date IN VARCHAR2, rbi_time IN VARCHAR2);
END intranet;
/

CREATE OR REPLACE PACKAGE BODY intranet AS
	FUNCTION get_procs (application VARCHAR2) RETURN ProcCurType IS
	procs		ProcCurType;
	BEGIN
		OPEN procs FOR
			SELECT DISTINCT object_name
			FROM all_arguments
			WHERE owner = 'WEBADMIN' 
			  and (object_name LIKE application || '%');
		RETURN procs;
	END get_procs;

	FUNCTION get_proc_args (application VARCHAR2) RETURN ProcArgCurType IS
	proc_args	ProcArgCurType;
	BEGIN
		OPEN proc_args FOR
			SELECT *
			FROM all_arguments
			WHERE owner = 'WEBADMIN' and object_name LIKE application || '%' AND in_out = 'IN'
			ORDER BY object_name, position;
		RETURN proc_args;
	END get_proc_args;

	FUNCTION helpdesk_callno_query RETURN NbrCurType IS
	id_no		NbrCurType;
	BEGIN
		OPEN id_no FOR
			SELECT helpdesk_callseq.NEXTVAL
			FROM dual;
		RETURN id_no;
	END;

	PROCEDURE get_time (time_string OUT VARCHAR2) IS
	BEGIN
		SELECT TO_CHAR(SYSDATE,'HH24:MM:SS') 
		INTO time_string
		FROM dual;
	END get_time;

	PROCEDURE get_date (date_string OUT VARCHAR2) IS
	BEGIN
		SELECT TO_CHAR(SYSDATE,'DD/MM/YYYY') 
		INTO date_string
		FROM dual;
	END get_date;

	FUNCTION helpdesk_time_query (dummy IN VARCHAR2) RETURN ProcCurType IS
	time_q		ProcCurType;
	BEGIN
		OPEN time_q FOR
		  SELECT TO_CHAR(SYSDATE,'HH24:MM:SS') FROM dual;
		RETURN time_q;
	END helpdesk_time_query;

	FUNCTION helpdesk_date_query (dummy IN VARCHAR2) RETURN ProcCurType IS
	date_q		ProcCurType;
	BEGIN
		OPEN date_q FOR
		  SELECT TO_CHAR(SYSDATE,'DD/MM/YYYY') FROM dual;
		RETURN date_q;
	END helpdesk_date_query;

	FUNCTION helpdesk_phone_query (phone_no IN VARCHAR2) RETURN UserRecCurType IS
	user		UserRecCurType;
	BEGIN
		OPEN user FOR
			SELECT firstname, surname, site
			FROM whitepages
			WHERE phone = phone_no;
		RETURN user;
	END helpdesk_phone_query;

	FUNCTION helpdesk_callinfo_query (choice IN NUMBER, helpdesk_dude IN VARCHAR2) RETURN CallinfoRecCurType IS
	call_rec		CallinfoRecCurType;
	BEGIN
		IF choice = 1 THEN
			OPEN call_rec FOR
				SELECT cl_nbr, cl_taken_date, cl_priority, CONCAT(CONCAT(firstname,' '),surname), cl_problem
				FROM calllog, whitepages
				WHERE cl_status != 'Completed' and phone (+) = cl_phone
				ORDER BY cl_priority, cl_taken_date;
		ELSIF choice = 2 THEN
			OPEN call_rec FOR
				SELECT cl_nbr, cl_taken_date, cl_priority, CONCAT(CONCAT(firstname,' '),surname), cl_problem
				FROM calllog, whitepages
				WHERE cl_status != 'Completed' AND cl_assigned_to = ' ' AND phone (+) = cl_phone
				ORDER BY cl_priority, cl_taken_date;
		ELSIF choice = 3 THEN
			OPEN call_rec FOR
				SELECT cl_nbr, cl_taken_date, cl_priority, CONCAT(CONCAT(firstname,' '),surname), cl_problem
				FROM calllog, whitepages
				WHERE cl_status != 'Completed' AND cl_assigned_to = helpdesk_dude AND phone (+) = cl_phone
				ORDER BY cl_priority, cl_taken_date;
		ELSIF choice = 4 THEN
			OPEN call_rec FOR
				SELECT cl_nbr, cl_taken_date, cl_priority, CONCAT(CONCAT(firstname,' '),surname), cl_problem
				FROM calllog, whitepages
				WHERE cl_status = 'Completed' AND phone (+) = cl_phone
				ORDER BY cl_priority, cl_taken_date;
		END IF;
		RETURN call_rec;
	END helpdesk_callinfo_query;

	FUNCTION helpdesk_calllog_query (log_no IN NUMBER) RETURN CalllogRecCurType IS
	call_rec	CalllogRecCurType;
	BEGIN
		OPEN call_rec FOR
			SELECT cl_nbr,cl_phone,firstname,surname,site,cl_priority,cl_category,cl_status,cl_assigned_to,cl_problem,
			       cl_solution,cl_taken_date,cl_taken_time,cl_taken_by,cl_assigned_date,cl_assigned_time,cl_assigned_by,
			       cl_completed_date,cl_completed_time,cl_comp_by FROM calllog, whitepages
			WHERE phone (+) = cl_phone AND cl_nbr = log_no
			ORDER BY cl_taken_date, cl_nbr;
		RETURN call_rec;
	END helpdesk_calllog_query;

	PROCEDURE helpdesk_phone_add (phone_no IN VARCHAR2, fname IN VARCHAR2, sname IN VARCHAR2, site_in IN NUMBER) IS
	BEGIN
		INSERT INTO whitepages (empno, surname, firstname, phone, site, official)
		VALUES (phone_empseq.NEXTVAL, sname, fname, phone_no, site_in, 'N');
	END helpdesk_phone_add;


	PROCEDURE helpdesk_calllog_add (log_no IN NUMBER, phone IN VARCHAR2, priority IN VARCHAR2, category IN VARCHAR2, 
		status IN VARCHAR2, assigned IN VARCHAR2, problem IN VARCHAR2, taken_by IN VARCHAR2) IS
	BEGIN
		INSERT INTO calllog 
			(cl_nbr,cl_phone,cl_priority,cl_category,cl_status,cl_assigned_to,cl_problem,cl_taken_date,cl_taken_time,
			cl_taken_by)
		VALUES (log_no, phone, priority, category, status, assigned, problem, TO_CHAR(SYSDATE,'DD/MM/YYYY'),
			TO_CHAR(SYSDATE,'HH24:MM:SS'),taken_by);
	END helpdesk_calllog_add;


	PROCEDURE helpdesk_calllog_update (nbr IN NUMBER, priority IN VARCHAR2, category IN VARCHAR2, status IN VARCHAR2, 
		problem IN VARCHAR2, solution IN VARCHAR2, assigned_date IN VARCHAR2, assigned_time IN VARCHAR2, 
		assigned_by IN VARCHAR2, assigned_to IN VARCHAR2, completed_date IN VARCHAR2, completed_time IN VARCHAR2,
		comp_by IN VARCHAR2) IS
	BEGIN
		UPDATE calllog 
		SET cl_priority = priority, cl_category = category, cl_status = status, cl_problem = problem,
			cl_solution = solution, cl_assigned_date = assigned_date, cl_assigned_time = assigned_time,
			cl_assigned_by = assigned_by, cl_assigned_to = assigned_to, cl_completed_date = completed_date, 
			cl_completed_time = completed_time, cl_comp_by = comp_by
		WHERE cl_nbr = nbr;
	END helpdesk_calllog_update;

	PROCEDURE helpdesk_calllog_delete (nbr IN NUMBER) IS
	BEGIN
		DELETE from calllog
		where cl_nbr = nbr;
	END helpdesk_calllog_delete;

	FUNCTION phone_query (choice IN NUMBER, search_string IN VARCHAR2) RETURN PhoneRecCurType IS
	query_results	PhoneRecCurType;
	BEGIN
		IF choice = 1 THEN
			OPEN query_results FOR
				SELECT * FROM whitepages
				WHERE UPPER (firstname) = UPPER (search_string)
				ORDER BY surname;
		ELSIF choice = 2 THEN
			OPEN query_results FOR
				SELECT * FROM whitepages
				WHERE UPPER (surname) = UPPER (search_string)
				ORDER BY surname;
		ELSIF choice = 3 THEN
			OPEN query_results FOR
				SELECT * FROM whitepages
				WHERE UPPER (firstname) like '%' || UPPER (search_string) || '%'
					OR UPPER (surname) like '%' || UPPER (search_string) || '%'
					OR UPPER (title) like '%' || UPPER (search_string) || '%'
					OR UPPER (phone) like '%' || UPPER (search_string) || '%'
					OR UPPER (mobile) like '%' || UPPER (search_string) || '%'
				ORDER BY surname;
		END IF;
		RETURN query_results;
	END phone_query;


	PROCEDURE phone_add (sname IN VARCHAR2, fname IN VARCHAR2, job_title IN VARCHAR2, phone_no IN VARCHAR2, 
		mobile_no in VARCHAR2, site_in IN NUMBER) IS
	BEGIN
		INSERT INTO whitepages (surname, firstname, title, phone, mobile, empno, site, official)
		VALUES (sname, fname, job_title, phone_no, mobile_no, phone_empseq.NEXTVAL, site_in, 'Y');
	END phone_add;


	PROCEDURE phone_update (nbr IN NUMBER, sname IN VARCHAR2, fname IN VARCHAR2, job_title IN VARCHAR2, phone_no IN VARCHAR2, 
		mobile_no in VARCHAR2, site_in IN VARCHAR2, offic IN VARCHAR2) IS
	BEGIN
		UPDATE whitepages
		SET surname = sname, firstname = fname, title = job_title, phone = phone_no, mobile = mobile_no, site = site_in, 
			official = offic
		WHERE empno = nbr;
	END phone_update;


	PROCEDURE phone_delete (nbr IN NUMBER) IS
	BEGIN
		DELETE FROM whitepages
		WHERE empno = nbr;
	END phone_delete;

--	**********************************************************************

	FUNCTION roombook_query (rbi_room IN VARCHAR2, rbi_date in VARCHAR2) RETURN RoomRecCurType IS
	room		RoomRecCurType;
	BEGIN
		OPEN room FOR
			SELECT rb_book_time, rb_message FROM roombook
			WHERE rb_room = rbi_room and rb_book_date = rbi_date;
		RETURN room;
	END roombook_query;

	PROCEDURE roombook_add (rbi_room IN VARCHAR2, rbi_date IN VARCHAR2, rbi_time IN VARCHAR2, rbi_mesg IN VARCHAR2) IS
	BEGIN
		INSERT INTO roombook (rb_room, rb_book_date, rb_book_time, rb_message)
		VALUES (rbi_room, rbi_date, rbi_time, rbi_mesg);
	END roombook_add;

	PROCEDURE roombook_update (rbi_room IN VARCHAR2, rbi_date IN VARCHAR2, rbi_time IN VARCHAR2, rbi_mesg IN VARCHAR2) IS
	BEGIN
		UPDATE roombook set rb_message = rbi_mesg
		where rb_room = rbi_room and rb_book_date = rbi_date and rb_book_time = rbi_time;
	END roombook_update;

	PROCEDURE roombook_delete (rbi_room IN VARCHAR2, rbi_date IN VARCHAR2, rbi_time IN VARCHAR2) IS
	BEGIN
		DELETE from roombook
		 where rb_room = rbi_room and rb_book_date = rbi_date and rb_book_time = rbi_time;
	END roombook_delete;

END intranet;
/

GRANT EXECUTE ON intranet TO intra_user;
CREATE PUBLIC SYNONYM intranet for intranet;
